@@ -1790,16 +1790,18 @@ play_file (gpointer callback_data)
 	GnomeVFSResult result;
 	GnomeVFSHandle *handle;
 	char *buffer;
+	const char *audio_device = NULL;
 	GnomeVFSFileSize bytes_read;
 
+	audio_device = g_getenv ("AUDIODEV");
 	icon_view = FM_ICON_VIEW (callback_data);
 	
 	file = icon_view->details->audio_preview_file;
 	file_uri = nautilus_file_get_uri (file);
 	mime_type = nautilus_file_get_mime_type (file);
 	is_mp3 = eel_strcasecmp (mime_type, "audio/mpeg") == 0;
-	is_ogg = eel_strcasecmp (mime_type, "application/x-ogg") == 0 ||
-                eel_strcasecmp (mime_type, "application/ogg") == 0;
+	is_ogg = eel_strcasecmp (mime_type, "application/ogg") == 0 ||
+                eel_strcasecmp (mime_type, "application/x-ogg") == 0;
 	
 	mp3_pid = fork ();
 	if (mp3_pid == (pid_t) 0) {
@@ -1816,7 +1818,11 @@ play_file (gpointer callback_data)
 			} else {
 				suffix += 1; /* skip the period */
 			}
-			command_str = g_strdup_printf("play -t %s -", suffix);
+			if (audio_device) {
+				command_str = g_strdup_printf("play -d %s -t %s -", audio_device, suffix);
+			} else {
+				command_str = g_strdup_printf("play -t %s -", suffix);
+			}
 		}
 
 		/* read the file with gnome-vfs, feeding it to the sound player's standard input */
@@ -1970,6 +1976,7 @@ icon_container_preview_callback (NautilusIconContainer *container,
 		mime_type = nautilus_file_get_mime_type (file);
 
 		if ((eel_istr_has_prefix (mime_type, "audio/")
+		     || eel_istr_has_prefix (mime_type, "application/ogg")
 		     || eel_istr_has_prefix (mime_type, "application/x-ogg"))
 		    && eel_strcasecmp (mime_type, "audio/x-pn-realaudio") != 0
 		    && eel_strcasecmp (mime_type, "audio/x-mpegurl") != 0
