@@ -26,6 +26,7 @@
 #include "fm-properties-window.h"
 
 #include "fm-error-reporting.h"
+#include <eel/eel-accessibility.h>
 #include <eel/eel-ellipsizing-label.h>
 #include <eel/eel-gdk-pixbuf-extensions.h>
 #include <eel/eel-glib-extensions.h>
@@ -61,6 +62,7 @@
 #include <libgnomeui/gnome-dialog.h>
 #include <libgnomeui/gnome-uidefs.h>
 #include <libgnomevfs/gnome-vfs.h>
+#include <libnautilus-extension/nautilus-property-page-provider.h>
 #include <libnautilus-private/nautilus-customization-data.h>
 #include <libnautilus-private/nautilus-entry.h>
 #include <libnautilus-private/nautilus-file-attributes.h>
@@ -70,6 +72,7 @@
 #include <libnautilus-private/nautilus-emblem-utils.h>
 #include <libnautilus-private/nautilus-link.h>
 #include <libnautilus-private/nautilus-metadata.h>
+#include <libnautilus-private/nautilus-module.h>
 #include <libnautilus-private/nautilus-undo-signal-handlers.h>
 #include <libnautilus-private/nautilus-mime-actions.h>
 #include <libnautilus-private/nautilus-view-identifier.h>
@@ -123,6 +126,8 @@ struct FMPropertiesWindowDetails {
 
 	guint total_count;
 	GnomeVFSFileSize total_size;
+
+	guint long_operation_underway;
 };
 
 enum {
@@ -148,18 +153,11 @@ enum {
 typedef struct {
 	GList *original_files;
 	GList *target_files;
-	FMDirectoryView *directory_view;
+	GtkWidget *parent_widget;
 	char *pending_key;
 	GHashTable *pending_files;
 } StartupData;
 
-typedef struct {
-	FMPropertiesWindow *window;
-	GtkWidget *vbox;
-	char *view_name;
-} ActivationData;
-
-
 /* drag and drop definitions */
 
 enum {
@@ -189,7 +187,7 @@ static void is_directory_ready_callback (NautilusFile *file,
 					 gpointer data);
 static void cancel_group_change_callback          (gpointer                 callback_data);
 static void cancel_owner_change_callback          (gpointer                 callback_data);
-static void directory_view_destroyed_callback     (FMDirectoryView         *view,
+static void parent_widget_destroyed_callback      (GtkWidget                *widget,
 						   gpointer                 callback_data);
 static void select_image_button_callback          (GtkWidget               *widget,
 						   FMPropertiesWindow      *properties_window);
@@ -201,7 +199,7 @@ static void remove_pending                        (StartupData             *data
 						   gboolean                 cancel_call_when_ready,
 						   gboolean                 cancel_timed_wait,
 						   gboolean                 cancel_destroy_handler);
-static void append_bonobo_pages                   (FMPropertiesWindow *window);
+static void append_extension_pages                (FMPropertiesWindow *window);
 
 GNOME_CLASS_BOILERPLATE (FMPropertiesWindow, fm_properties_window,
 			 GtkWindow, GTK_TYPE_WINDOW);
@@ -267,21 +265,40 @@ get_target_file_for_original_file (NautilusFile *file)
 {
 	NautilusFile *target_file;
 	char *uri_to_display;
+	GnomeVFSVolume *volume;
+	GnomeVFSDrive *drive;
 	NautilusDesktopLink *link;
 
 	target_file = NULL;
-	if (NAUTILUS_IS_DESKTOP_ICON_FILE (file)) {
+	if (nautilus_file_has_volume (file)) {
+		volume = nautilus_file_get_volume (file);
+		if (volume != NULL) {
+			uri_to_display = gnome_vfs_volume_get_activation_uri (volume);
+			target_file = nautilus_file_get (uri_to_display);
+			g_free (uri_to_display);
+		}
+		gnome_vfs_volume_unref (volume);
+	} else if (nautilus_file_has_drive (file)) {
+		drive = nautilus_file_get_drive (file);
+		if (drive != NULL) {
+			uri_to_display = gnome_vfs_drive_get_activation_uri (drive);
+			target_file = nautilus_file_get (uri_to_display);
+			g_free (uri_to_display);
+		}
+		gnome_vfs_drive_unref (drive);
+	} else if (NAUTILUS_IS_DESKTOP_ICON_FILE (file)) {
 		link = nautilus_desktop_icon_file_get_link (NAUTILUS_DESKTOP_ICON_FILE (file));
 		
 		/* map to linked URI for these types of links */
 		uri_to_display = nautilus_desktop_link_get_activation_uri (link);
 		if (uri_to_display) {
-			target_file = nautilus_file_get (uri_to_display);
-			g_free (uri_to_display);
-		}
+                        target_file = nautilus_file_get (uri_to_display);
+                        g_free (uri_to_display);
+                }
 		
 		g_object_unref (link);
-	}
+        }
+
 
 	if (target_file != NULL) {
 		return target_file;
@@ -380,6 +397,8 @@ update_properties_window_icon (GtkImage *image)
 	pixbuf = get_pixbuf_for_properties_window (window);
 
 	gtk_image_set_from_pixbuf (image, pixbuf);
+
+	gtk_window_set_icon (GTK_WINDOW (window), pixbuf);
 	
 	g_object_unref (pixbuf);
 }
@@ -455,8 +474,8 @@ fm_properties_window_drag_data_received (GtkWidget *widget, GdkDragContext *cont
 
 	if (!exactly_one) {
 		eel_show_error_dialog
-			(_("You can't assign more than one custom icon at a time! "
-			   "Please drag just one image to set a custom icon."), 
+			(_("You can't assign more than one custom icon at a time!"),
+			 _("Please drag just one image to set a custom icon."), 
 			 _("More Than One Image"),
 			 window);
 	} else {		
@@ -466,15 +485,15 @@ fm_properties_window_drag_data_received (GtkWidget *widget, GdkDragContext *cont
 		} else {	
 			if (eel_is_remote_uri (uris[0])) {
 				eel_show_error_dialog
-					(_("The file that you dropped is not local.  "
-					   "You can only use local images as custom icons."), 
+					(_("The file that you dropped is not local."),
+					 _("You can only use local images as custom icons."), 
 					 _("Local Images Only"),
 					 window);
 				
 			} else {
 				eel_show_error_dialog
-					(_("The file that you dropped is not an image.  "
-					   "You can only use local images as custom icons."),
+					(_("The file that you dropped is not an image."),
+					 _("You can only use local images as custom icons."),
 					 _("Images Only"),
 					 window);
 			}
@@ -905,7 +924,7 @@ update_properties_window_title (FMPropertiesWindow *window)
 }
 
 static void
-clear_bonobo_pages (FMPropertiesWindow *window)
+clear_extension_pages (FMPropertiesWindow *window)
 {
 	int i;
 	int num_pages;
@@ -914,11 +933,11 @@ clear_bonobo_pages (FMPropertiesWindow *window)
 	num_pages = gtk_notebook_get_n_pages
 				(GTK_NOTEBOOK (window->details->notebook));
 
-	for (i=0; i <  num_pages; i++) {
+	for (i = 0; i < num_pages; i++) {
 		page = gtk_notebook_get_nth_page
 				(GTK_NOTEBOOK (window->details->notebook), i);
 
-		if (g_object_get_data (G_OBJECT (page), "is-bonobo-page")) {
+		if (g_object_get_data (G_OBJECT (page), "is-extension-page")) {
 			gtk_notebook_remove_page
 				(GTK_NOTEBOOK (window->details->notebook), i);
 			num_pages--;
@@ -928,10 +947,10 @@ clear_bonobo_pages (FMPropertiesWindow *window)
 }
 
 static void
-refresh_bonobo_pages (FMPropertiesWindow *window)
+refresh_extension_pages (FMPropertiesWindow *window)
 {
-	clear_bonobo_pages (window);
-	append_bonobo_pages (window);	
+	clear_extension_pages (window);
+	append_extension_pages (window);	
 }
 
 static void
@@ -974,8 +993,8 @@ remove_from_dialog (FMPropertiesWindow *window,
 					      G_CALLBACK (properties_window_update),
 					      window);
 
-	nautilus_file_monitor_remove (original_file, window);
-	nautilus_file_monitor_remove (target_file, window);
+	nautilus_file_monitor_remove (original_file, &window->details->original_files);
+	nautilus_file_monitor_remove (target_file, &window->details->target_files);
 
 	nautilus_file_unref (original_file);
 	nautilus_file_unref (target_file);
@@ -1062,7 +1081,7 @@ properties_window_update (FMPropertiesWindow *window,
 		window->details->mime_list = mime_list;
 	} else {
 		if (!mime_list_equal (window->details->mime_list, mime_list)) {
-			refresh_bonobo_pages (window);			
+			refresh_extension_pages (window);			
 		}
 		
 		eel_g_list_free_deep (window->details->mime_list);
@@ -1357,7 +1376,7 @@ activate_group_callback (GtkMenuItem *menu_item, FileNamePair *pair)
 		(cancel_group_change_callback,
 		 pair->file,
 		 _("Cancel Group Change?"),
-		 _("Changing group"),
+		 _("Changing group."),
 		 NULL); /* FIXME bugzilla.gnome.org 42397: Parent this? */
 	nautilus_file_set_group
 		(pair->file, pair->name,
@@ -1519,7 +1538,7 @@ activate_owner_callback (GtkMenuItem *menu_item, FileNamePair *pair)
 		(cancel_owner_change_callback,
 		 pair->file,
 		 _("Cancel Owner Change?"),
-		 _("Changing owner"),
+		 _("Changing owner."),
 		 NULL); /* FIXME bugzilla.gnome.org 42397: Parent this? */
 	nautilus_file_set_owner
 		(pair->file, pair->name,
@@ -1613,7 +1632,7 @@ synch_user_menu (GtkOptionMenu *option_menu, NautilusFile *file)
 	eel_g_list_free_deep (users);
 }	
 
-static void
+static GtkOptionMenu*
 attach_owner_menu (GtkTable *table,
 		   int row,
 		   NautilusFile *file)
@@ -1628,6 +1647,7 @@ attach_owner_menu (GtkTable *table,
 	g_signal_connect_object (file, "changed",
 				 G_CALLBACK (synch_user_menu),
 				 option_menu, G_CONNECT_SWAPPED);	
+	return option_menu;
 }
 
 static guint
@@ -1736,13 +1756,10 @@ directory_contents_value_field_update (FMPropertiesWindow *window)
 	} else {
 		char *size_str;
 		size_str = gnome_vfs_format_file_size_for_display (total_size);
-		if (total_count == 1) {
-			text = g_strdup_printf (_("1 item, with size %s"),
-						size_str);
-		} else {
-			text = g_strdup_printf (_("%d items, totalling %s"),
-						total_count, size_str);
-		}
+		text = g_strdup_printf (ngettext("%d item, with size %s",
+						 "%d items, totalling %s",
+						 total_count),
+					total_count, size_str);
 		g_free (size_str);
 
 		if (unreadable_directory_count != 0) {
@@ -2440,6 +2457,11 @@ permission_change_callback (NautilusFile *file, GnomeVFSResult result, gpointer
 	g_assert (callback_data != NULL);
 
 	window = FM_PROPERTIES_WINDOW (callback_data);
+	if (window->details->long_operation_underway == 1) {
+		/* finished !! */
+		gdk_window_set_cursor (GTK_WIDGET (window)->window, NULL);
+	}
+	window->details->long_operation_underway--;
 	
 	/* Report the error if it's an error. */
 	fm_report_error_setting_permissions (file, result, NULL);
@@ -2517,6 +2539,17 @@ permission_button_toggled (GtkToggleButton *button,
 					   G_CALLBACK (permission_button_toggled),
 					   window);
 
+	if (window->details->long_operation_underway == 0) {
+		/* start long operation */
+		GdkCursor * cursor;
+	       
+		cursor = gdk_cursor_new (GDK_WATCH);
+		gdk_window_set_cursor (GTK_WIDGET (window)->window, cursor);
+		gdk_cursor_unref (cursor);
+	}
+	window->details->long_operation_underway += g_list_length (files_on);
+	window->details->long_operation_underway += g_list_length (files_off);
+
 	for (l = files_on; l != NULL; l = l->next) {
 		NautilusFile *file;
 		
@@ -2635,10 +2668,12 @@ static void
 add_permissions_checkbox (FMPropertiesWindow *window,
 			  GtkTable *table, 
 			  int row, int column, 
-			  GnomeVFSFilePermissions permission_to_check)
+			  GnomeVFSFilePermissions permission_to_check,
+			  GtkLabel *label_for)
 {
 	GtkWidget *check_button;
 	gchar *label;
+	gboolean a11y_enabled;
 
 	if (column == PERMISSIONS_CHECKBOXES_READ_COLUMN) {
 		label = _("_Read");
@@ -2659,6 +2694,12 @@ add_permissions_checkbox (FMPropertiesWindow *window,
 	set_up_permissions_checkbox (window, 
 				     check_button, 
 				     permission_to_check);
+
+	a11y_enabled = GTK_IS_ACCESSIBLE (gtk_widget_get_accessible (check_button));
+	if (a11y_enabled) {
+		eel_accessibility_set_up_label_widget_relation (GTK_WIDGET (label_for),
+								check_button);
+	}
 }
 
 static GtkWidget *
@@ -2812,7 +2853,12 @@ create_permissions_page (FMPropertiesWindow *window)
 	guint last_row;
 	guint checkbox_titles_row;
 	GtkLabel *group_label;
+	GtkLabel *owner_label;
+	GtkLabel *owner_perm_label;
+	GtkLabel *group_perm_label;
+	GtkLabel *other_perm_label;
 	GtkOptionMenu *group_menu;
+	GtkOptionMenu *owner_menu;
 	GList *file_list;
 
 	vbox = create_page_with_vbox (window->details->notebook,
@@ -2839,13 +2885,14 @@ create_permissions_page (FMPropertiesWindow *window)
 				    GTK_WIDGET (page_table), 
 				    TRUE, TRUE, 0);
 
-		attach_title_field (page_table, last_row, _("File owner:"));
-
-			
 		if (!is_multi_file_window (window) && nautilus_file_can_set_owner (get_target_file (window))) {
+			owner_label = attach_title_field (page_table, last_row, _("File _owner:"));
 			/* Option menu in this case. */
-			attach_owner_menu (page_table, last_row, get_target_file (window));
+			owner_menu = attach_owner_menu (page_table, last_row, get_target_file (window));
+			gtk_label_set_mnemonic_widget (owner_label,
+						       GTK_WIDGET (owner_menu));
 		} else {
+			attach_title_field (page_table, last_row, _("File owner:"));
 			/* Static text in this case. */
 			attach_value_field (window, 
 					    page_table, last_row, VALUE_COLUMN,
@@ -2861,7 +2908,7 @@ create_permissions_page (FMPropertiesWindow *window)
 			/* Option menu in this case. */
 			group_menu = attach_group_menu (page_table, last_row,
 							get_target_file (window));
-			gtk_label_set_mnemonic_widget (GTK_LABEL (group_label),
+			gtk_label_set_mnemonic_widget (group_label,
 						       GTK_WIDGET (group_menu));
 		} else {
 			last_row = append_title_field (page_table,
@@ -2877,9 +2924,9 @@ create_permissions_page (FMPropertiesWindow *window)
 
 		append_separator (page_table);
 		
-		checkbox_titles_row = append_title_field (page_table, _("Owner:"), NULL);
-		append_title_field (page_table, _("Group:"), NULL);
-		append_title_field (page_table, _("Others:"), NULL);
+		checkbox_titles_row = append_title_field (page_table, _("Owner:"), &owner_perm_label);
+		append_title_field (page_table, _("Group:"), &group_perm_label);
+		append_title_field (page_table, _("Others:"), &other_perm_label);
 		
 		check_button_table = GTK_TABLE (gtk_table_new 
 						   (PERMISSIONS_CHECKBOXES_ROW_COUNT, 
@@ -2897,55 +2944,64 @@ create_permissions_page (FMPropertiesWindow *window)
 					  check_button_table, 
 					  PERMISSIONS_CHECKBOXES_OWNER_ROW,
 					  PERMISSIONS_CHECKBOXES_READ_COLUMN,
-					  GNOME_VFS_PERM_USER_READ);
+					  GNOME_VFS_PERM_USER_READ,
+					  owner_perm_label);
 
 		add_permissions_checkbox (window,
 					  check_button_table, 
 					  PERMISSIONS_CHECKBOXES_OWNER_ROW,
 					  PERMISSIONS_CHECKBOXES_WRITE_COLUMN,
-					  GNOME_VFS_PERM_USER_WRITE);
+					  GNOME_VFS_PERM_USER_WRITE,
+					  owner_perm_label);
 
 		add_permissions_checkbox (window,
 					  check_button_table, 
 					  PERMISSIONS_CHECKBOXES_OWNER_ROW,
 					  PERMISSIONS_CHECKBOXES_EXECUTE_COLUMN,
-					  GNOME_VFS_PERM_USER_EXEC);
+					  GNOME_VFS_PERM_USER_EXEC,
+					  owner_perm_label);
 
 		add_permissions_checkbox (window,
 					  check_button_table, 
 					  PERMISSIONS_CHECKBOXES_GROUP_ROW,
 					  PERMISSIONS_CHECKBOXES_READ_COLUMN,
-					  GNOME_VFS_PERM_GROUP_READ);
+					  GNOME_VFS_PERM_GROUP_READ,
+					  group_perm_label);
 
 		add_permissions_checkbox (window,
 					  check_button_table, 
 					  PERMISSIONS_CHECKBOXES_GROUP_ROW,
 					  PERMISSIONS_CHECKBOXES_WRITE_COLUMN,
-					  GNOME_VFS_PERM_GROUP_WRITE);
+					  GNOME_VFS_PERM_GROUP_WRITE,
+					  group_perm_label);
 
 		add_permissions_checkbox (window,
 					  check_button_table, 
 					  PERMISSIONS_CHECKBOXES_GROUP_ROW,
 					  PERMISSIONS_CHECKBOXES_EXECUTE_COLUMN,
-					  GNOME_VFS_PERM_GROUP_EXEC);
+					  GNOME_VFS_PERM_GROUP_EXEC,
+					  group_perm_label);
 
 		add_permissions_checkbox (window,
 					  check_button_table, 
 					  PERMISSIONS_CHECKBOXES_OTHERS_ROW,
 					  PERMISSIONS_CHECKBOXES_READ_COLUMN,
-					  GNOME_VFS_PERM_OTHER_READ);
+					  GNOME_VFS_PERM_OTHER_READ,
+					  other_perm_label);
 
 		add_permissions_checkbox (window,
 					  check_button_table, 
 					  PERMISSIONS_CHECKBOXES_OTHERS_ROW,
 					  PERMISSIONS_CHECKBOXES_WRITE_COLUMN,
-					  GNOME_VFS_PERM_OTHER_WRITE);
+					  GNOME_VFS_PERM_OTHER_WRITE,
+					  other_perm_label);
 
 		add_permissions_checkbox (window,
 					  check_button_table, 
 					  PERMISSIONS_CHECKBOXES_OTHERS_ROW,
 					  PERMISSIONS_CHECKBOXES_EXECUTE_COLUMN,
-					  GNOME_VFS_PERM_OTHER_EXEC);
+					  GNOME_VFS_PERM_OTHER_EXEC,
+					  other_perm_label);
 
 		append_separator (page_table);
 
@@ -2977,209 +3033,52 @@ create_permissions_page (FMPropertiesWindow *window)
 	}
 }
 
-static GtkWidget *
-bonobo_page_error_message (const char *view_name,
-			   const char *msg)
+static void
+append_extension_pages (FMPropertiesWindow *window)
 {
-	GtkWidget *hbox;
-	GtkWidget *label;
-	GtkWidget *image;
-
-	hbox = gtk_hbox_new (FALSE, GNOME_PAD);
-	image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_ERROR,
-					  GTK_ICON_SIZE_DIALOG);
-
-	msg = g_strdup_printf ("There was an error while trying to create the view named `%s':  %s", view_name, msg);
-	label = gtk_label_new (msg);
-
-	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
-
-	gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-	gtk_widget_show_all (hbox);
-
-	return hbox;
-}
-
-static CORBA_sequence_CORBA_string *
-get_uri_list (GList *file_list)
-{
-	CORBA_sequence_CORBA_string *uri_list;
-	GList *l;
-	int i;
+	GList *providers;
+	GList *p;
 	
-	uri_list = CORBA_sequence_CORBA_string__alloc ();
-	uri_list->_maximum = g_list_length (file_list);
-	uri_list->_length = uri_list->_maximum;
-	uri_list->_buffer = CORBA_sequence_CORBA_string_allocbuf (uri_list->_length);
-	for (i = 0, l = file_list; l != NULL; i++, l = l->next) {
-		char *uri;
+ 	providers = nautilus_module_get_extensions_for_type (NAUTILUS_TYPE_PROPERTY_PAGE_PROVIDER);
+	
+	for (p = providers; p != NULL; p = p->next) {
+		NautilusPropertyPageProvider *provider;
+		GList *pages;
+		GList *l;
+
+		provider = NAUTILUS_PROPERTY_PAGE_PROVIDER (p->data);
 		
-		uri = nautilus_file_get_uri (NAUTILUS_FILE (l->data));
-		uri_list->_buffer[i] = CORBA_string_dup (uri);
-		g_free (uri);
-	}	
+		pages = nautilus_property_page_provider_get_pages 
+			(provider, window->details->target_files);
+		
+		for (l = pages; l != NULL; l = l->next) {
+			NautilusPropertyPage *page;
+			GtkWidget *page_widget;
+			GtkWidget *label;
+			
+			page = NAUTILUS_PROPERTY_PAGE (l->data);
 
-	return uri_list;
-}
+			g_object_get (G_OBJECT (page), 
+				      "page", &page_widget, "label", &label, 
+				      NULL);
+			
+			gtk_notebook_append_page (window->details->notebook, 
+						  page_widget, label);
 
-static void
-bonobo_page_activate_callback (CORBA_Object obj,
-			       const char *error_reason,
-			       gpointer user_data)
-{
-	ActivationData *data;
-	FMPropertiesWindow *window;
-	GtkWidget *widget;
-	CORBA_Environment ev;
+			g_object_set_data (G_OBJECT (page), 
+					   "is-extension-page",
+					   page);
 
-	data = (ActivationData *)user_data;
-	window = data->window;
+			gtk_widget_unref (page_widget);
+			gtk_widget_unref (label);
 
-	g_return_if_fail (FM_IS_PROPERTIES_WINDOW (window));
-
-	CORBA_exception_init (&ev);
-	widget = NULL;
-
-	if (obj != CORBA_OBJECT_NIL) {
-		Bonobo_Control control;
-		Bonobo_PropertyBag pb;
-		GList *keys;
-
-		control = Bonobo_Unknown_queryInterface
-				(obj, "IDL:Bonobo/Control:1.0", &ev);
-
-		pb = Bonobo_Control_getProperties (control, &ev);
-
-		if (!BONOBO_EX (&ev)) {
-			gboolean new_property;
-
-			keys = bonobo_pbclient_get_keys (pb, NULL);
-			new_property = FALSE;
-			if (g_list_find_custom (keys, "uris", (GCompareFunc)strcmp)) {
-				new_property = TRUE;
-			}
-			bonobo_pbclient_free_keys (keys);
-
-			if (new_property) {
-				/* Set the 'uris' property. */
-				CORBA_sequence_CORBA_string *uri_list;
-				BonoboArg *arg;
-				
-				uri_list = get_uri_list (window->details->target_files);
-				arg = bonobo_arg_new (TC_CORBA_sequence_CORBA_string);
-				arg->_value = uri_list;
-				bonobo_pbclient_set_value_async (pb, "uris", arg, &ev);
-				bonobo_arg_release (arg);
-			} else {
-				/* Set the 'URI' property. */
-				BonoboArg *arg;
-				char *uri;
-
-				if (is_multi_file_window (window)) {
-					g_warning ("Multifile property page does not support the 'uris' property");
-					bonobo_object_release_unref (pb, NULL);
-					bonobo_object_release_unref (control, NULL);
-					return;
-				}
-
-				uri = nautilus_file_get_uri (get_target_file (window));
-
-				arg = bonobo_arg_new (BONOBO_ARG_STRING);
-				BONOBO_ARG_SET_STRING (arg, uri);
-				bonobo_pbclient_set_value_async (pb, "URI", arg, &ev);
-				bonobo_arg_release (arg);
-				g_free (uri);
-			}
-
-			bonobo_object_release_unref (pb, NULL);
-
-			if (!BONOBO_EX (&ev)) {
-				widget = bonobo_widget_new_control_from_objref
-							(control, CORBA_OBJECT_NIL);
-				bonobo_object_release_unref (control, &ev);
-			}
+			g_object_unref (page);
 		}
+
+		g_list_free (pages);
 	}
 
-	if (widget == NULL) {
-		widget = bonobo_page_error_message (data->view_name,
-						    error_reason);
-	}
-
-	gtk_container_add (GTK_CONTAINER (data->vbox), widget);
-	gtk_widget_show (widget);
-
-	g_free (data->view_name);
-	g_free (data);
-}
-
-static gboolean
-can_handle_multiple_files (Bonobo_ServerInfo *info)
-{
-        Bonobo_ActivationProperty *prop;
- 
-        prop = bonobo_server_info_prop_find (info, "nautilus:can_handle_multiple_files");
-
-        return prop && prop->v._u.value_boolean;
-}
-
-static void
-append_bonobo_pages (FMPropertiesWindow *window)
-{
-	GList *all_components, *l;
-	GList *components;
-	CORBA_Environment ev;
-
-	/* find all the property pages for this file */
-	all_components = nautilus_mime_get_property_components_for_files
-		(window->details->target_files);
-	
-	/* filter out property pages that don't support multiple files */
-	if (is_multi_file_window (window)) {
-		components = NULL;
-		for (l = all_components; l != NULL; l = l->next) {
-			if (can_handle_multiple_files (l->data)) {
-				components = g_list_prepend (components, 
-							     l->data);
-			}
-		}
-		components = g_list_reverse (components);
-		g_list_free (all_components);
-	} else {
-		components = all_components;
-	}
-
-	CORBA_exception_init (&ev);
-
-	l = components;
-	while (l != NULL) {
-		NautilusViewIdentifier *view_id;
-		Bonobo_ServerInfo *server;
-		ActivationData *data;
-		GtkWidget *vbox;
-
-		server = l->data;
-		l = l->next;
-
-		view_id = nautilus_view_identifier_new_from_property_page (server);
-		vbox = create_page_with_vbox (window->details->notebook,
-					      view_id->name);
-
-		/* just a tag...the value doesn't matter */
-		g_object_set_data (G_OBJECT (vbox), "is-bonobo-page",
-				  vbox);
-
-		data = g_new (ActivationData, 1);
-		data->window = window;
-		data->vbox = vbox;
-		data->view_name = g_strdup (view_id->name);
-
-		bonobo_activation_activate_from_id_async (view_id->iid,
-					0, bonobo_page_activate_callback,
-					data, &ev);
-	}
-
+	nautilus_module_extension_list_free (providers);
 }
 
 static gboolean
@@ -3244,7 +3143,7 @@ static StartupData *
 startup_data_new (GList *original_files, 
 		  GList *target_files,
 		  const char *pending_key,
-		  FMDirectoryView *directory_view)
+		  GtkWidget *parent_widget)
 {
 	StartupData *data;
 	GList *l;
@@ -3252,7 +3151,7 @@ startup_data_new (GList *original_files,
 	data = g_new0 (StartupData, 1);
 	data->original_files = nautilus_file_list_copy (original_files);
 	data->target_files = nautilus_file_list_copy (target_files);
-	data->directory_view = directory_view;
+	data->parent_widget = parent_widget;
 	data->pending_key = g_strdup (pending_key);
 	data->pending_files = g_hash_table_new (g_direct_hash,
 						g_direct_equal);
@@ -3278,19 +3177,15 @@ static void
 help_button_callback (GtkWidget *widget, GtkWidget *property_window)
 {
 	GError *error = NULL;
-	char *message;
 
-	egg_help_display_desktop_on_screen (NULL, "user-guide", "wgosnautilus.xml", "gosnautilus-51",
+	egg_help_display_desktop_on_screen (NULL, "user-guide", "user-guide.xml", "gosnautilus-51",
 					    gtk_window_get_screen (GTK_WINDOW (property_window)),
 &error);
 
 	if (error) {
-		message = g_strdup_printf (_("There was an error displaying help: \n%s"),
-					   error->message);
-		eel_show_error_dialog (message, _("Couldn't show help"),
+		eel_show_error_dialog (_("There was an error displaying help."), error->message, _("Couldn't Show Help"),
 				       GTK_WINDOW (property_window));
 		g_error_free (error);
-		g_free (message);
 	}
 }
 
@@ -3312,7 +3207,7 @@ create_properties_window (StartupData *startup_data)
 	gtk_window_set_wmclass (GTK_WINDOW (window), "file_properties", "Nautilus");
 	gtk_window_set_resizable (GTK_WINDOW (window), FALSE);
 	gtk_window_set_screen (GTK_WINDOW (window),
-			       gtk_widget_get_screen (GTK_WIDGET (startup_data->directory_view)));
+			       gtk_widget_get_screen (startup_data->parent_widget));
 
 	/* Set initial window title */
 	update_properties_window_title (window);
@@ -3329,10 +3224,11 @@ create_properties_window (StartupData *startup_data)
 		file = NAUTILUS_FILE (l->data);
 
 		attributes = nautilus_icon_factory_get_required_file_attributes ();
-		attributes |= NAUTILUS_FILE_ATTRIBUTE_DISPLAY_NAME;
+		attributes |= NAUTILUS_FILE_ATTRIBUTE_DISPLAY_NAME
+			| NAUTILUS_FILE_ATTRIBUTE_SLOW_MIME_TYPE;
 
 		nautilus_file_monitor_add (NAUTILUS_FILE (l->data),
-					   window, 
+					   &window->details->original_files, 
 					   attributes);	
 	}
 	
@@ -3348,7 +3244,7 @@ create_properties_window (StartupData *startup_data)
 		}
 		
 		attributes |= NAUTILUS_FILE_ATTRIBUTE_METADATA;
-		nautilus_file_monitor_add (file, window, attributes);
+		nautilus_file_monitor_add (file, &window->details->target_files, attributes);
 	}	
 		
 	for (l = window->details->target_files; l != NULL; l = l->next) {
@@ -3392,7 +3288,7 @@ create_properties_window (StartupData *startup_data)
 	}
 
 	/* append pages from available views */
-	append_bonobo_pages (window);
+	append_extension_pages (window);
 
 	/* Create box for help and close buttons. */
 	hbox = gtk_hbutton_box_new ();
@@ -3483,9 +3379,9 @@ cancel_create_properties_window_callback (gpointer callback_data)
 }
 
 static void
-directory_view_destroyed_callback (FMDirectoryView *view, gpointer callback_data)
+parent_widget_destroyed_callback (GtkWidget *widget, gpointer callback_data)
 {
-	g_assert (view == ((StartupData *)callback_data)->directory_view);
+	g_assert (widget == ((StartupData *)callback_data)->parent_widget);
 	
 	remove_pending ((StartupData *)callback_data, TRUE, TRUE, FALSE);
 }
@@ -3518,8 +3414,8 @@ remove_pending (StartupData *startup_data,
 			(cancel_create_properties_window_callback, startup_data);
 	}
 	if (cancel_destroy_handler) {
-		g_signal_handlers_disconnect_by_func (startup_data->directory_view,
-						      G_CALLBACK (directory_view_destroyed_callback),
+		g_signal_handlers_disconnect_by_func (startup_data->parent_widget,
+						      G_CALLBACK (parent_widget_destroyed_callback),
 						      startup_data);
 	}
 
@@ -3561,7 +3457,7 @@ is_directory_ready_callback (NautilusFile *file,
 
 void
 fm_properties_window_present (GList *original_files,
-			      FMDirectoryView *directory_view) 
+			      GtkWidget *parent_widget) 
 {
 	GList *l, *next;
 	GtkWidget *parent_window;
@@ -3571,7 +3467,7 @@ fm_properties_window_present (GList *original_files,
 	char *pending_key;
 
 	g_return_if_fail (original_files != NULL);
-	g_return_if_fail (FM_IS_DIRECTORY_VIEW (directory_view));
+	g_return_if_fail (GTK_IS_WIDGET (parent_widget));
 
 	/* Create the hash tables first time through. */
 	if (windows == NULL) {
@@ -3588,7 +3484,7 @@ fm_properties_window_present (GList *original_files,
 	existing_window = get_existing_window (original_files);
 	if (existing_window != NULL) {
 		gtk_window_set_screen (existing_window,
-				       gtk_widget_get_screen (GTK_WIDGET (directory_view)));
+				       gtk_widget_get_screen (parent_widget));
 		gtk_window_present (existing_window);
 		return;
 	}
@@ -3606,25 +3502,26 @@ fm_properties_window_present (GList *original_files,
 	startup_data = startup_data_new (original_files, 
 					 target_files,
 					 pending_key,
-					 directory_view);
+					 parent_widget);
 
 	nautilus_file_list_free (target_files);
+	g_free(pending_key);
 
 	/* Wait until we can tell whether it's a directory before showing, since
 	 * some one-time layout decisions depend on that info. 
 	 */
 	
-	g_hash_table_insert (pending_lists, pending_key, pending_key);
-	g_signal_connect (directory_view, "destroy",
-			  G_CALLBACK (directory_view_destroyed_callback), startup_data);
+	g_hash_table_insert (pending_lists, startup_data->pending_key, startup_data->pending_key);
+	g_signal_connect (parent_widget, "destroy",
+			  G_CALLBACK (parent_widget_destroyed_callback), startup_data);
 
-	parent_window = gtk_widget_get_ancestor (GTK_WIDGET (directory_view), GTK_TYPE_WINDOW);
+	parent_window = gtk_widget_get_ancestor (parent_widget, GTK_TYPE_WINDOW);
 
 	eel_timed_wait_start
 		(cancel_create_properties_window_callback,
 		 startup_data,
 		 _("Cancel Showing Properties Window?"),
-		 _("Creating Properties window"),
+		 _("Creating Properties window."),
 		 parent_window == NULL ? NULL : GTK_WINDOW (parent_window));
 
 
@@ -3649,13 +3546,13 @@ real_destroy (GtkObject *object)
 	remove_window (window);
 
 	for (l = window->details->original_files; l != NULL; l = l->next) {
-		nautilus_file_monitor_remove (NAUTILUS_FILE (l->data), window);
+		nautilus_file_monitor_remove (NAUTILUS_FILE (l->data), &window->details->original_files);
 	}
 	nautilus_file_list_free (window->details->original_files);
 	window->details->original_files = NULL;
 	
 	for (l = window->details->target_files; l != NULL; l = l->next) {
-		nautilus_file_monitor_remove (NAUTILUS_FILE (l->data), window);
+		nautilus_file_monitor_remove (NAUTILUS_FILE (l->data), &window->details->target_files);
 	}
 	nautilus_file_list_free (window->details->target_files);
 	window->details->target_files = NULL;
